{***************************************************************}
{*            RichView wrapper for QuickReport                 *}
{***************************************************************}
{*                                                             *}
{* Copyright (c) 2005-2011                                     *}
{*               Henk van Bokhorst / IVS Systemhouse B.V.      *}
{*             & QBS Software                                  *}
{* All rights reserved                                         *}
{*                                                             *}
{***************************************************************}
{*                                                             *}
{* Unit: RVQRUtils                                             *}
{*                                                             *}
{* Various routines used in the other units                    *}
{*                                                             *}
{* 02/15/2011 First released version                           *}
{*                                                             *}
{***************************************************************}

unit RVQRUtils;

interface

uses Windows, Classes, SysUtils;

function IsRVF(TextStream: TMemoryStream): boolean;

{ Encode / decode RVF file in AsciiHex }
procedure DecodeRVF(Encoded: string; Decoded: TMemoryStream);
procedure EncodeRVF(Decoded, Encoded: TMemoryStream);

implementation

{ Check to see if a stream contains rvf content or not (rtf) }

function IsRVF(TextStream: TMemoryStream): boolean;

var Buffer: array[0..4] of Byte;

begin
  TextStream.Position := 0;
  TextStream.Read(Buffer, SizeOf(Buffer));
  Result := ((Buffer[0] = 45) and (Buffer[1] = 56) and (Buffer[2] = 32) and   // RVF files start
             (Buffer[3] = 49) and (Buffer[4] = 32));                          // with '-8 1 '
end;

{ Encode / decode RVF file in AsciiHex }

procedure DecodeRVF(Encoded: string; Decoded: TMemoryStream);

var
  Buffer: array [0..999] of Byte;
  StringPos, BufferPos: LongInt;

  function HexToByte(Hex1, Hex2: char): Byte;
  begin
    Result := 0;
    if (Hex1 <= '9') and (Hex1 >= '0')
    then Result := 16 * (Ord(Hex1) - Ord('0'))
    else
      if (Hex1 >= 'A') and (Hex1 <= 'F')
      then Result := 16 * (Ord(Hex1) - Ord('A') + 10);

    if (Hex2 <= '9') and (Hex2 >= '0')
    then Result := Result + (Ord(Hex2) - Ord('0'))
    else
      if (Hex2 >= 'A') and (Hex2 <= 'F')
      then Result := Result + (Ord(Hex2) - Ord('A') + 10);
  end;

begin
  StringPos := 1;
  BufferPos := 0;
  while StringPos <= Length(Encoded) do
  begin
    Buffer[BufferPos] := HexToByte(Encoded[StringPos], Encoded[StringPos + 1]);
    if BufferPos = 999
    then
    begin
      Decoded.WriteBuffer(Buffer, 1000);
      BufferPos := 0;
    end
    else Inc(BufferPos);
    Inc(StringPos, 2);
  end;
  if BufferPos > 0 then Decoded.WriteBuffer(Buffer, BufferPos);
end;

procedure EncodeRVF(Decoded, Encoded: TMemoryStream);

const REC_SIZE = 2048;

var
  Buffer: array [0..REC_SIZE] of Byte;
  WasRead, Loop: Integer;
  BufString: string;

  function ByteToHex(B: Byte): String;

    function Hex(B: Byte): Char;
    begin
      if B > 9 then Result := Chr(B + Ord('A') - 10) else Result := Chr(B + Ord('0'))
    end;

  begin
    Result := Hex(B shr 4) + Hex(B and $f);
  end;

begin
  Encoded.Clear;
  Decoded.Position := 0;
  WasRead          := Decoded.Read(Buffer, REC_SIZE);
  while WasRead > 0 do
  begin
    BufString := '';
    for Loop := 0 to WasRead - 1 do BufString := BufString + ByteToHex(Buffer[Loop]);

    Encoded.Write(BufString[1], Length(BufString));
    WasRead := Decoded.Read(Buffer, REC_SIZE);
  end;
  Encoded.Write(BufString[1], Length(BufString));
end;

end.
